// Copyright 1999 (c) by Perforce Software, Inc.  All rights reserved.
//
// p4wEditProcessorView:
//   The Edit processor view.


// -------------------------------------
// Includes
//
#include <stdlib.h>
#include <string.h>

#include <p4wp4.h>

#include "p4wEditProcessorView.h"

#include "p4wCmdPane.h"

//
// -------------------------------------
// Constructors and destructor.
//
p4wEditProcessorView::p4wEditProcessorView(p4wRequest & Request)
	: p4wRunView(Request)
{
	//
	// Parse formAction to see if this is a redisplay request

	ProcessFormAction();		
	fAc = AC_EDITFRM;
	
	//
	// Set the short and the long versions of the title.
	// If there is no POST data the title is "Error".
	
	if( !ProcessForm() ) {
		fShortTitle.Set( "Error" );
		fFullTitle.Set( "Error" );
		return;
	}
		
	fShortTitle.Set( "Command Result" );
	fFullTitle.Set( "Edit Result" );
}

p4wEditProcessorView::~p4wEditProcessorView()
{
}

// -------------------------------------
// Control item render functions.
//

void p4wEditProcessorView::RenderContent()
{
	//
	// Issue edit command (passing the results through a 
	// CmdPane).

	StrBuf fileType;
	
	// Build a spec to parse the edit options

	if( !ProcessForm() ) {
		RenderError("Requested URL is invalid in this context.");
		return;
	}

	//
	// Parse options and build up the command args

	title.Set("Edit ");

	if( specTable.GetVar("OptionChg") ) {
		ArgAndTitle( "-c" );
		ArgAndTitle( *specTable.GetVar("changelistVal") );
	}

	if( specTable.GetVar("OptionFt") ) {
		ArgAndTitle( "-t" );
		GetFileType( fileType );
		ArgAndTitle( fileType );
	}

	//
	// Process the files. If this returned an error don't issue command

	if( !AddFileArgs() ) 
		return;
	
	// Run edit

	p4wCmdPane cmdPane(*this, fRequest, title.Text());
	fRequest.p4("edit", 0, 0, &cmdPane);
	fRequest.p4Wait();
}

