// Copyright 1999 (c) by Perforce Software, Inc.  All rights reserved.
//
// p4wRevertProcessorView:
//   The Revert processor view.


// -------------------------------------
// Includes
//
#include <stdlib.h>
#include <string.h>

#include <p4wp4.h>

#include "p4wRevertProcessorView.h"

#include "p4wCmdPane.h"
#include "p4wPreviewPane.h"

//
// -------------------------------------
// Constructors and destructor.
//
p4wRevertProcessorView::p4wRevertProcessorView(p4wRequest & Request)
	: p4wRunView(Request)
{
	//
	// Parse formAction to see if this is a redisplay request

	ProcessFormAction();		
	fAc = AC_REVERTFRM;
	
	//
	// Set the short and the long versions of the title.
	// If there is no POST data the title is "Error".
	
	if( !ProcessForm() ) {
		fShortTitle.Set( "Error" );
		fFullTitle.Set( "Error" );
		return;
	}
		
	fShortTitle.Set( "Command Result" );
	
	if( fPreview )
		fFullTitle.Set( "Revert Preview Result" );
	else
		fFullTitle.Set( "Revert Result" );
}

p4wRevertProcessorView::~p4wRevertProcessorView()
{
}

// -------------------------------------
// Control item render functions.
//

void p4wRevertProcessorView::RenderContent()
{
	//
	// Issue revert command (passing the results through a 
	// CmdPane).

	// Build a spec to parse the revert options

	if( !ProcessForm() ) {
		RenderError("Requested URL is invalid in this context.");
		return;
	}

	//
	// Determine which edit options were chosen

	//
	// Parse options and build up the command args

	title.Set("Revert ");
	
	if( fPreview ) 
		ArgAndTitle( "-n" );
		
	if( specTable.GetVar("OptionUnch") ) {
		ArgAndTitle( "-a" );
	}

	if( specTable.GetVar("OptionChg") ) {
		ArgAndTitle( "-c" );
		ArgAndTitle( *specTable.GetVar("changelistVal") );
	}
	
	//
	// Process the files. If this returned an error, don't issue the command
	
	if( !AddFileArgs() ) 
		return;

	//
	// Run.

	if( fPreview ) {
		p4wPreviewPane prevPane( *this, fRequest, title.Text() );
		fRequest.p4( "revert", 0, 0, &prevPane );
		fRequest.p4Wait();
	} else {
		p4wCmdPane cmdPane( *this, fRequest, title.Text() );
		fRequest.p4( "revert", 0, 0, &cmdPane );
		fRequest.p4Wait();
	}
}

